/* This file is part of I.S.A.A.C.S.

   I.S.A.A.C.S. is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

    I.S.A.A.C.S. is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with I.S.A.A.C.S.  If not, see <http://www.gnu.org/licenses/>. */

#if defined (HAVE_CONFIG_H)
#  include <config.h>
#endif

#include <string.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#ifdef G_OS_WIN32
#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#  include <shellapi.h>
#endif

#include "global.h"
#include "curve.h"
#include "callbacks.h"
#include "interface.h"
#include "grcall.h"
#include "bdcall.h"
#include "angcall.h"
#include "ringscall.h"
#include "spcall.h"
#include "bvcall.h"

/* Make url/email in about dialog clickable */
void about_dialog_handle_url (GtkAboutDialog *dialog, const gchar *link, gpointer data)
{
  GtkWidget * error_dialog;
  gchar * url, * escaped;

  g_return_if_fail (GTK_IS_ABOUT_DIALOG (dialog));
  g_return_if_fail (link != NULL);

/* simple check if this is an email address */
  if (!g_str_has_prefix (link, "mailto:") && strchr (link, '@') != NULL)
  {
    escaped = g_uri_escape_string (link, NULL, FALSE);
    url = g_strdup_printf ("mailto:%s", escaped);
    g_free (escaped);
  }
  else
  {
    url = g_strdup (link);
  }

#ifdef G_OS_WIN32
  HINSTANCE h;
  h = ShellExecute( NULL, "open", url, NULL, NULL, SW_SHOWNORMAL );
  if ((int)h <= 32)
#else
  GError    *error = NULL;
  GdkScreen *screen;
  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
  gtk_show_uri (screen, url, gtk_get_current_event_time (), &error);
  if (error != NULL)
#endif
  {
    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
                                           GTK_DIALOG_DESTROY_WITH_PARENT,
                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
                                           "Failed to open browser to show:\n%s", url);
    gtk_dialog_run (GTK_DIALOG (error_dialog));
    gtk_widget_destroy (error_dialog);
  }
  g_free (url);
}

G_MODULE_EXPORT void create_aboutdialog (void)
{
  GtkWidget * aboutdialog;
  GdkPixbuf * logoisaacs = gdk_pixbuf_new_from_file(PACKAGE_LOGO, NULL);
  const gchar *authors[] = {"Dr. Sébastien Le Roux <sebastien.leroux@ipcms.unistra.fr>",
                            "Dr. Valeri Petkov <petko1vg@cmich.edu>",
                            "",
                            "Work on the project was supported by NSF",
                            "Mixed Network Former Effect - MNFE project",
                            "DMR Grant No. 0710564.",
                            NULL};
  const gchar * weburl = "http://isaacs.sourceforge.net/";
  const gchar * weblabel = "Website of the project";
  const gchar * comments = "Structural properties of 3D models\n"
                           "obtained using computer simulations.";
  const gchar * copyrights = "Copyright © 2009-2018\nSébastien Le Roux\nValeri Petkov";
  const gchar * thelicense = "I.S.A.A.C.S. is free software: you can redistribute it and/or modify\n"
    "it under the terms of the GNU General Public License as published by\n"
    "the Free Software Foundation, either version 3 of the License, or\n"
    "(at your option) any later version.\n\n"
    "I.S.A.A.C.S. is distributed in the hope that it will be useful,\n"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
    "GNU General Public License for more details.\n\n"
    "You should have received a copy of the GNU General Public License\n"
    "along with I.S.A.A.C.S.";

  gtk_about_dialog_set_url_hook (about_dialog_handle_url, NULL, NULL);
  gtk_about_dialog_set_email_hook (about_dialog_handle_url, NULL, NULL);

  aboutdialog = gtk_about_dialog_new ();
  gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(aboutdialog), logoisaacs);
  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG(aboutdialog), VERSION);
  gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG(aboutdialog), PACKAGE);
  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(aboutdialog), comments);
  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG(aboutdialog), weburl);
  gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG(aboutdialog), weblabel);
  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG(aboutdialog), authors);
  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG(aboutdialog), copyrights);
  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG(aboutdialog), thelicense);
  gtk_about_dialog_set_wrap_license (GTK_ABOUT_DIALOG(aboutdialog), FALSE);
  gtk_window_set_icon (GTK_WINDOW (aboutdialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(aboutdialog));
  gtk_widget_destroy(aboutdialog);
}

void show_info (char * information)
{
  GtkWidget * dialog;

  dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
            GTK_MESSAGE_INFO, GTK_BUTTONS_OK, information, "title");
  gtk_window_set_title(GTK_WINDOW(dialog), "Information");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
}

void show_info_ (double * valdij)
{
  GtkWidget * dialog;
  char * itot=NULL;
  char * val;
  char deb[105]="No bond length distribution to be displayed\n"
                "Only one inter-atomic distance in the system\n"
                "at Dij = ";
  char end[4]=" Å";

  val=g_strdup_printf("%lf", * valdij);
  itot = g_malloc((10 + strlen(deb)+strlen(val)+strlen(end))*sizeof*itot);
  strcpy(itot, deb);
  strcat(itot, val);
  strcat(itot, end);
  dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
            GTK_MESSAGE_INFO, GTK_BUTTONS_OK, itot, "title");
  gtk_window_set_title(GTK_WINDOW(dialog), "Information");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
  g_free (val);
  g_free (itot);
}

GtkWidget * show_pop (char * pop, GtkWidget * pwin)
{
  GtkWidget * wpop;
  GtkWidget * label;
  GtkWidget * hpop;
  GtkWidget * imgpop;

  wpop = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (wpop, 600, 80);
  gtk_window_set_position (GTK_WINDOW (wpop), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (wpop), FALSE);
  gtk_window_set_transient_for (GTK_WINDOW (wpop), GTK_WINDOW (pwin));
  gtk_window_set_title(GTK_WINDOW(wpop), "Information");
  gtk_window_set_icon (GTK_WINDOW (wpop), THEMOL);

  hpop = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hpop);
  gtk_container_add (GTK_CONTAINER (wpop), hpop);
  imgpop = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_BUTTON);
//  gtk_widget_set_size_request (imgpop, 20, 20);
  gtk_widget_show (imgpop);
  gtk_box_pack_start (GTK_BOX (hpop), imgpop, TRUE, TRUE, 0);
  label = gtk_label_new (pop);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hpop), label, TRUE, TRUE, 0);

  return (wpop);
}

GtkWidget * show_popup (char * popup, GtkWidget * pwin)
{
  GtkWidget * wpopup;
  GtkWidget * label;
  GtkWidget * vpop;
  GtkWidget * hpop;
  GtkWidget * imgpop;

  wpopup = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (wpopup, 600, 80);
  gtk_window_set_position (GTK_WINDOW (wpopup), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (wpopup), FALSE);
  gtk_window_set_transient_for (GTK_WINDOW (wpopup), GTK_WINDOW (pwin));
  gtk_window_set_title (GTK_WINDOW(wpopup), "Information");
  gtk_window_set_icon (GTK_WINDOW (wpopup), THEMOL);

  vpop = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vpop);
  gtk_container_add (GTK_CONTAINER (wpopup), vpop);

  hpop = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hpop);
  gtk_box_pack_start (GTK_BOX (vpop), hpop, TRUE, TRUE, 0);

  imgpop = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_BUTTON);
//  gtk_widget_set_size_request (imgpop, 20, 20);
  gtk_widget_show (imgpop);
  gtk_box_pack_start (GTK_BOX (hpop), imgpop, TRUE, TRUE, 0);
  label = gtk_label_new (popup);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hpop), label, TRUE, TRUE, 0);

  progressbar = gtk_progress_bar_new ();
  gtk_widget_show (progressbar);
  gtk_box_pack_start (GTK_BOX (vpop), progressbar, FALSE, TRUE, 0);
  gtk_widget_set_size_request (progressbar, -1, 30);

  return (wpopup);
}

void show_warning (char * warning)
{
  GtkWidget * dialog;
  dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, warning, "title");
  gtk_window_set_title(GTK_WINDOW(dialog), "Warning");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
}

void show_warning_ (char * warning, char * sub, char * tab)
{
  /* This function is called from fortran 90 */

  GtkWidget * dialog;
  char * wtot=NULL;

  wtot = g_strdup_printf("%s%s%s%s%s", warning, retl, sub, retl, tab);
  dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, wtot, "title");
  gtk_window_set_title(GTK_WINDOW(dialog), "Information");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
  g_free (wtot);
}

void show_error (char * error)
{
  GtkWidget * dialog;
  char * etot=NULL;
  char ifbug[40]="If this is a bug please report it to:\n";
  char open[2]="<";
  char close[2]=">";

  etot = g_strdup_printf("%s%s%s%s%s%s", error, retl, ifbug, open, PACKAGE_BUGREPORT, close);
  dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
            GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, etot, "title");
  gtk_window_set_title(GTK_WINDOW(dialog), "Error");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
  g_free(etot);
}

void show_error_ (char * error, char * sub, char * tab)
{
  /* This function is called from fortran 90 */

  GtkWidget * dialog;
  char * etot=NULL;
  char ifbug[40]="If this is a bug please report it to:\n";
  char open[2]="<";
  char close[2]=">";
  etot = g_strdup_printf("%s%s%s%s%s%s%s%s%s%s",
         error, retl, sub, retl, tab, retl, ifbug, open, PACKAGE_BUGREPORT, close);
  dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, etot, "title");
  gtk_window_set_title(GTK_WINDOW(dialog), "Error");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
  g_free(etot);
}

G_MODULE_EXPORT void show_question (GtkWidget *widget)
{
  GtkWidget * dialog;
  GtkWidget * info;
  gboolean displaymain;

  dialog = gtk_message_dialog_new (NULL,
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_MESSAGE_QUESTION,
            GTK_BUTTONS_YES_NO,
            "Are you sure you want to quit ?");
  gtk_window_set_title(GTK_WINDOW(dialog), "Leaving I.S.A.A.C.S.");
  gtk_window_set_icon (GTK_WINDOW (dialog), THEMOL);
  gint result = gtk_dialog_run(GTK_DIALOG (dialog));
  displaymain=FALSE;
  switch (result)
  {
    case GTK_RESPONSE_YES:
      on_Quit_activate();
      break;
    case GTK_RESPONSE_NO:
      displaymain=TRUE;
      break;
  }
  gtk_widget_destroy (dialog);
  if (displaymain)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "MainWindow");
    gtk_widget_show_all(info);
  }
}

void initcombospec_ ()
{
  int i;
  GtkFixed * boxfixed;

  /* Création de la GtkComboBox des espèces chimiques */
  specbox = gtk_combo_box_new_text();
  for ( i=0 ; i < nspec ; i++ )
  {
    gtk_combo_box_append_text (GTK_COMBO_BOX(specbox), label[i]);
  }
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "BoxFixed");
  gtk_fixed_put(boxfixed, specbox, -1, -1);
  gtk_widget_set_sensitive(specbox, 1);
  g_signal_connect(G_OBJECT(specbox), "changed", G_CALLBACK(on_Species_get), NULL);
  gtk_widget_show(specbox);
  gtk_combo_box_set_active(GTK_COMBO_BOX(specbox), 0);
}

void initview_ (int * nats, int * nspc, int * stps)
{
  GtkLabel * info;
  char * str;

  natomes = * nats;
  str=g_strdup_printf("%d",natomes);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "Natomes");
  gtk_label_set_text (info, str);
  g_free(str);

  nspec = * nspc;
  str=g_strdup_printf("%d",nspec);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "Nspecies");
  gtk_label_set_text (info, str);
  g_free(str);

  steps = * stps;
  str=g_strdup_printf("%d",steps);
  info = (GtkLabel *) gtk_builder_get_object (BuilderGtk, "Nstep");
  gtk_label_set_text (info, str);
  g_free(str);
}

void cheminfo_ (int prop[nspec], double * volume, double * density,
                                 double * fvolume, double * ecfvolume)
{
  GtkWidget * info;
  GtkTextBuffer * buffer;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  int i;
  char start[44]="\n\t    Project successfully validated !\n";
  char sep[61]="\n********************************************************\n";
  char load[16]="\nFile loaded: ";
  char type[16]="\nFile type:   ";
  char na[21]="Number of atoms   = ";
  char ns[21]="Number of species = ";
  char vol[18]="Volume         = ";
  char den[18]="Density        = ";
  char nden[18]="Number density = ";
  char ndunit[15]=" Atoms / Å^3\n";
  char vunit[8]=" Å^3\n";
  char fvol[18]="Free volume    = ";
  char efvol[6]=" +/- ";
  char pfvol[5]=" %\n";
  char dunit[12]=" g / cm^3\n";
  char chem[33]="\tBasic chemistry information:\n";
  char form[21]="Empirical formula:\t";
  char line[55]="----------------------------------------------------\n";
  char ats[63]= "Atom\t Number    Fraction\tNumber density (atoms / Å^3)\n";
  char atr[28]="Atom\t Number    Fraction\n";
  char vrads[48]="Calculated using the following atomic radii:\n";
  char vrad[49]="Calculated using the following atomic radius:\n";
  char uvun[6]=" Å\n";
  char * cptot;
  char * tcptot;
  char rtab[4]=":\t";
  PangoFontDescription  *font_desc;

  tcptot = g_strdup_printf("%s%s%s%s%s%s%s%s%s%s%s%s%s%s%d%s%s%d%s",
                            sep, start, sep, retl, load, coordfile, retl,
                            type, typefile, retl, retl, chem, line, na,
                            natomes, retl, ns, nspec, retl);
  if ( *volume != 0.0 )
  {
    cptot=g_strdup_printf("%s%s%s%14.6lf%s%s%14.6lf%s%s%14.6lf%s%s",
                          tcptot, retl, vol, *volume, vunit, den, *density,
                          dunit, nden, natomes/ *volume, ndunit, retl);
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
    if (steps > 1)
    {
      cptot=g_strdup_printf("%s%s%14.6lf%s%lf%s%s%14.6lf%s%lf%s%s",
                    tcptot, fvol, *fvolume, efvol, *ecfvolume, vunit, fvol,
                    100*(*fvolume)/(*volume), efvol, 100*(*ecfvolume)/(*volume), pfvol, retl);
      g_free(tcptot);
      tcptot=g_strdup_printf("%s", cptot);
      g_free(cptot);
    }
    else
    {
      cptot=g_strdup_printf("%s%s%lf%s%s%lf%s%s",
      tcptot, fvol, *fvolume, vunit, fvol, 100*(*fvolume)/(*volume), pfvol, retl);
      g_free(tcptot);
      tcptot=g_strdup_printf("%s", cptot);
      g_free(cptot);
    }
    if (nspec == 1)
    {
      cptot=g_strdup_printf("%s%s", tcptot, vrad);
    }
    else
    {
      cptot=g_strdup_printf("%s%s", tcptot, vrads);
    }
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
    for ( i=0 ; i< nspec ; i++ )
    {
      cptot=g_strdup_printf("%s%s%s%lf%s", tcptot, label[i], rtab, rad[i], uvun);
      tcptot=g_strdup_printf("%s", cptot);
      g_free(cptot);
    }
    cptot=g_strdup_printf("%s%s", tcptot, retl);
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
  }
  cptot=g_strdup_printf("%s%s", tcptot, form);
  g_free(tcptot);
  tcptot=g_strdup_printf("%s", cptot);
  g_free(cptot);
  for ( i=0 ; i < nspec ; i++ )
  {
    cptot=g_strdup_printf("%s%s", tcptot, label[i]);
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
    if (prop[i] != 1)
    {
      cptot=g_strdup_printf("%s%d", tcptot, prop[i]);
      g_free(tcptot);
      tcptot=g_strdup_printf("%s", cptot);
      g_free(cptot);
    }
    cptot=g_strdup_printf("%s%s", tcptot, " ");
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
  }
  cptot=g_strdup_printf("%s%s%s", tcptot, retl, retl);
  g_free(tcptot);
  tcptot=g_strdup_printf("%s", cptot);
  g_free(cptot);
  if ( *volume != 0.0 )
  {
	cptot=g_strdup_printf("%s%s", tcptot, ats);
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
  }
  else
  {
	cptot=g_strdup_printf("%s%s", tcptot, atr);
    g_free(tcptot);
    tcptot=g_strdup_printf("%s", cptot);
    g_free(cptot);
  }
  g_print("%s", tcptot);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  font_desc = pango_font_description_from_string ("monospace 8");
  gtk_widget_modify_font (info, font_desc);
  pango_font_description_free (font_desc);
  buffer = gtk_text_buffer_new(NULL);
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_delete(buffer, &bStart, &bEnd);
  gtk_text_buffer_insert (buffer, &bStart, tcptot, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(tcptot);
}

void display_cheminfo_ (int * as, int * ns, int * is, double * xs, double * vs)
{
  GtkWidget * info;
  GtkTextBuffer * buffer;
  GtkTextIter bStart;
  GtkTextIter bEnd;
  char * citot;
  char * tcitot;
  char tab[3]="\t";


  tcitot=g_strdup_printf("%s%s%7d%s%11.6lf", label[*as], tab, *ns, tab, *xs);
  if (*vs != 0.0)
  {
    citot=g_strdup_printf("%s%s%11.6lf", tcitot, tab, *vs);
    g_free(tcitot);
    tcitot=g_strdup_printf("%s", citot);
    g_free(citot);
  }
  citot=g_strdup_printf("%s%s", tcitot, retl);
  g_free(tcitot);
  g_print("%s", citot);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "TextInfo");
  buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(info));
  gtk_text_buffer_get_start_iter (buffer, &bStart);
  gtk_text_buffer_get_end_iter (buffer, &bEnd);
  gtk_text_buffer_insert (buffer, &bEnd, citot, -1);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW(info), buffer);
  g_free(citot);
}

void turnactions (int * val, int * gsv)
{
  GtkWidget * info;

  if (* gsv != 1)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rdf");
    gtk_action_set_sensitive(GTK_ACTION(info), * val);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hgr");
    gtk_widget_set_sensitive(info, * val);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqkpoints");
    gtk_action_set_sensitive(GTK_ACTION(info), * val);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsk");
    gtk_widget_set_sensitive(info, * val);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsko");
    gtk_widget_set_sensitive(info, * val);
  }
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfftrdf");
  gtk_action_set_sensitive(GTK_ACTION(info), 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsq");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grfft");
  gtk_action_set_sensitive(GTK_ACTION(info), 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hgq");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Bonds");
  gtk_action_set_sensitive(GTK_ACTION(info), * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hbd");
  gtk_widget_set_sensitive(info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Angles");
  gtk_action_set_sensitive(GTK_ACTION(info), * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hang");
  gtk_widget_set_sensitive(info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Rings");
  gtk_action_set_sensitive(GTK_ACTION(info), * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hri");
  gtk_widget_set_sensitive(info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Spheric");
  gtk_action_set_sensitive(GTK_ACTION(info), 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hsp");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Valence");
  gtk_action_set_sensitive(GTK_ACTION(info), 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hbv");
  gtk_widget_set_sensitive(info, 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Msd");
  gtk_action_set_sensitive(GTK_ACTION(info), 0);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hmsd");
  gtk_widget_set_sensitive(info, 0);
  if (steps > 1 && * val ==1)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Msd");
    gtk_action_set_sensitive(GTK_ACTION(info), * val);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "button-hmsd");
    gtk_widget_set_sensitive(info, * val);
  }
  if (* val == 0)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rdfgr");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqfft");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqkpoint");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqfft");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bonds");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angles");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "msd");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "rings");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spheric");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "valence");
    if (GTK_WIDGET_VISIBLE(info)) gtk_widget_hide(info);
  }
}

void turnchecks (int * val)
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "grcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "sqcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "skcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "gqcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "bdcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "angcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "ringcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "spcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "msdcheck");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispmsd");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgr");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsq");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsk");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispgq");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispbd");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispang");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "disprings");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "dispsp");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(info), * val);
  if (GTK_WIDGET_SENSITIVE (info)) gtk_widget_set_sensitive (info, * val);
}

void initcwidgets ()
{
  numgr = 8+5*nspec*nspec;
  numsq = 8+4*nspec*nspec;
  numsk = 8+4*nspec*nspec;
  numgq = numgr;
  numbd = nspec*nspec;
  numan = nspec*nspec*nspec+nspec*nspec*nspec*nspec;
  nummsd = 0;
  if (steps > 1) nummsd = 14*nspec+6;
  numri = 20*(nspec+1);
  numsh = nspec+1;
  if (nspec == 2)
  {
    numgr = numgr + 6;
    numsq = numsq + 8;
    numsk = numsk + 8;
    numgq = numgq + 6;
  }
  numwid = numgr + numsq + numsk + numgq + numbd + numan + numri + numsh + nummsd;
  run = TRUE;
  fixedgtkgr = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedRDF");
  tablegtkgr = GTK_TABLE(gtk_table_new (numgr, 1, TRUE));
  gtk_fixed_put(fixedgtkgr, GTK_WIDGET(tablegtkgr), -1, -1);
  fixedgtksq = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedSQ");
  tablegtksq = GTK_TABLE(gtk_table_new (numsq, 1, TRUE));
  gtk_fixed_put(fixedgtksq, GTK_WIDGET(tablegtksq), -1, -1);
  fixedgtksk = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedSK");
  tablegtksk = GTK_TABLE(gtk_table_new (numsk, 1, TRUE));
  gtk_fixed_put(fixedgtksk, GTK_WIDGET(tablegtksk), -1, -1);
  fixedgtkgq = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedGQ");
  tablegtkgq = GTK_TABLE(gtk_table_new (numgr, 1, TRUE));
  gtk_fixed_put(fixedgtkgq, GTK_WIDGET(tablegtkgq), -1, -1);
  fixedgtkbd = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedBD");
  tablegtkbd = GTK_TABLE(gtk_table_new (numbd, 1, TRUE));
  gtk_fixed_put(fixedgtkbd, GTK_WIDGET(tablegtkbd), -1, -1);
  fixedgtkan = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedANG");
  tablegtkan = GTK_TABLE(gtk_table_new (numan, 1, TRUE));
  gtk_fixed_put(fixedgtkan, GTK_WIDGET(tablegtkan), -1, -1);
  fixedgtkri = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedrings");
  tablegtkri = GTK_TABLE(gtk_table_new (numri, 1, TRUE));
  gtk_fixed_put(fixedgtkri, GTK_WIDGET(tablegtkri), -1, -1);
  fixedgtksh = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedSH");
  tablegtksh = GTK_TABLE(gtk_table_new (numsh, 1, TRUE));
  gtk_fixed_put(fixedgtksh, GTK_WIDGET(tablegtksh), -1, -1);
  if (steps > 1)
  {
    fixedgtkms = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixedmsd");
    tablegtkms = GTK_TABLE(gtk_table_new (nummsd, 1, TRUE));
    gtk_fixed_put(fixedgtkms, GTK_WIDGET(tablegtkms), -1, -1);
  }

  shbox = calloc(nspec, sizeof*shbox);
  bvbox = calloc(nspec, sizeof*bvbox);
  curves = malloc(numwid*sizeof*curves);
  int i;
  for (i = 0; i<numwid; i++)
  {
    curves[i]=calloc(1, sizeof*curves[i]);
  }
}

void initbonds()
{
  int i;
  GtkFixed * boxfixed;

  trun = TRUE;
  bspeca = gtk_combo_box_new_text();
  bspecb = gtk_combo_box_new_text();
  aspeca = gtk_combo_box_new_text();
  aspecb = gtk_combo_box_new_text();
  rispec = gtk_combo_box_new_text();
  rspeca = gtk_combo_box_new_text();
  rspecb = gtk_combo_box_new_text();
  shspec = gtk_combo_box_new_text();
  bvspec = gtk_combo_box_new_text();
  gtk_combo_box_append_text (GTK_COMBO_BOX(rispec), "All");
  for ( i=0 ; i < nspec ; i++ )
  {
    gtk_combo_box_append_text (GTK_COMBO_BOX(bspeca), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(bspecb), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(aspeca), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(aspecb), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(rispec), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(rspeca), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(rspecb), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(shspec), label[i]);
    gtk_combo_box_append_text (GTK_COMBO_BOX(bvspec), label[i]);
  }
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-c1");
  gtk_fixed_put(boxfixed, bspeca, -1, -1);
  gtk_widget_set_sensitive(bspeca, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-c2");
  gtk_fixed_put(boxfixed, bspecb, -1, -1);
  gtk_widget_set_sensitive(bspecb, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-a1");
  gtk_fixed_put(boxfixed, aspeca, -1, -1);
  gtk_widget_set_sensitive(aspeca, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-a2");
  gtk_fixed_put(boxfixed, aspecb, -1, -1);
  gtk_widget_set_sensitive(aspecb, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-ring");
  gtk_fixed_put(boxfixed, rispec, -1, -1);
  gtk_widget_set_sensitive(rispec, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-r1");
  gtk_fixed_put(boxfixed, rspeca, -1, -1);
  gtk_widget_set_sensitive(rspeca, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-r2");
  gtk_fixed_put(boxfixed, rspecb, -1, -1);
  gtk_widget_set_sensitive(rspecb, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-s1");
  gtk_fixed_put(boxfixed, shspec, -1, -1);
  gtk_widget_set_sensitive(shspec, 1);
  boxfixed = (GtkFixed *) gtk_builder_get_object (BuilderGtk, "fixed-b1");
  gtk_fixed_put(boxfixed, bvspec, -1, -1);
  gtk_widget_set_sensitive(bvspec, 1);
  gtk_combo_box_set_active(GTK_COMBO_BOX(bspeca), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(bspecb), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(aspeca), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(aspecb), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(rspeca), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(rspecb), 0);
  gtk_combo_box_set_active(GTK_COMBO_BOX(rispec), 0);
  g_signal_connect(G_OBJECT(bspeca), "changed", G_CALLBACK(on_bspec_get), NULL);
  g_signal_connect(G_OBJECT(bspecb), "changed", G_CALLBACK(on_bspec_get), NULL);
  g_signal_connect(G_OBJECT(aspeca), "changed", G_CALLBACK(on_aspec_get), NULL);
  g_signal_connect(G_OBJECT(aspecb), "changed", G_CALLBACK(on_aspec_get), NULL);
  g_signal_connect(G_OBJECT(rspeca), "changed", G_CALLBACK(on_rspec_get), NULL);
  g_signal_connect(G_OBJECT(rspecb), "changed", G_CALLBACK(on_rspec_get), NULL);
  g_signal_connect(G_OBJECT(rispec), "changed", G_CALLBACK(on_rispec_get), NULL);
  g_signal_connect(G_OBJECT(shspec), "changed", G_CALLBACK(on_shspec_get), NULL);
  g_signal_connect(G_OBJECT(bvspec), "changed", G_CALLBACK(on_bvspec_get), NULL);
  gtk_widget_show(bspeca);
  gtk_widget_show(bspecb);
  gtk_widget_show(aspeca);
  gtk_widget_show(aspecb);
  gtk_widget_show(rspeca);
  gtk_widget_show(rspecb);
  gtk_widget_show(rispec);
  gtk_widget_show(shspec);
  gtk_widget_show(bvspec);
}

void setprogress_ (gdouble * prog)
{
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progressbar), * prog);
  while (gtk_events_pending()) gtk_main_iteration();
}

void setcalcs (int * status)
{
  GtkWidget * info;

  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcgr");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcsq");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcsk");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcgq");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcbonds");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcangles");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcrings");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcspheric");
  gtk_widget_set_sensitive (info, * status);
  info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcvalence");
  gtk_widget_set_sensitive (info, * status);
  if (steps > 1)
  {
    info = (GtkWidget *) gtk_builder_get_object (BuilderGtk, "Calcmsd");
    gtk_widget_set_sensitive(info, * status);
  }
}

int iask (char * question)
{
  GtkWidget *iask;
  GtkWidget *dialog_ask;
  GtkWidget *hboxa;
  GtkWidget *quest;
  GtkWidget *answer;
  GtkWidget *ok;
  const gchar * data;
  int val;

  iask = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (iask), 5);
  gtk_window_set_type_hint (GTK_WINDOW (iask), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_resizable (GTK_WINDOW (iask), FALSE);
  gtk_dialog_set_has_separator (GTK_DIALOG (iask), FALSE);
  gtk_window_set_icon (GTK_WINDOW (iask), THEMOL);
  gtk_window_set_title (GTK_WINDOW(iask), "Please enter");

  dialog_ask = GTK_DIALOG (iask)->vbox;
  gtk_box_set_spacing (GTK_BOX(dialog_ask), 15);

  hboxa = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (dialog_ask), hboxa, TRUE, TRUE, 0);
  gtk_widget_show(hboxa);
  quest = gtk_label_new (question);
  gtk_box_pack_start (GTK_BOX (hboxa), quest, TRUE, TRUE, 0);
  gtk_widget_show(quest);
  answer = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hboxa), answer, TRUE, TRUE, 0);
  gtk_widget_set_size_request (answer, 100, -1);
  gtk_entry_set_alignment (GTK_ENTRY(answer), 1.0);
  gtk_widget_show(answer);
  dialog_ask = GTK_DIALOG (iask)->action_area;
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_ask), GTK_BUTTONBOX_END);

  ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_dialog_add_action_widget (GTK_DIALOG (iask), ok, 1);
  gtk_widget_show(ok);

  if (gtk_dialog_run (GTK_DIALOG (iask)) == 1)
  {
    data = gtk_entry_get_text (GTK_ENTRY(answer));
    val = atof(data);
    gtk_widget_destroy (iask);
    return (val);
  }
  else
  {
    gtk_widget_destroy (iask);
    return (0);
  }
}

char * cask (char * question)
{
  GtkWidget *cask;
  GtkWidget *dialog_ask;
  GtkWidget *hboxa;
  GtkWidget *quest;
  GtkWidget *answer;
  GtkWidget *ok;
  const gchar * val;
  char * res;

  cask = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (cask), 5);
  gtk_window_set_type_hint (GTK_WINDOW (cask), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_resizable (GTK_WINDOW (cask), FALSE);
  gtk_dialog_set_has_separator (GTK_DIALOG (cask), FALSE);
  gtk_window_set_title (GTK_WINDOW(cask), "Please enter");
  gtk_window_set_icon (GTK_WINDOW (cask), THEMOL);

  dialog_ask = GTK_DIALOG (cask)->vbox;
  gtk_box_set_spacing (GTK_BOX(dialog_ask), 15);

  hboxa = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (dialog_ask), hboxa, TRUE, TRUE, 0);
  gtk_widget_show(hboxa);
  quest = gtk_label_new (question);
  gtk_box_pack_start (GTK_BOX (hboxa), quest, TRUE, TRUE, 0);
  gtk_widget_show(quest);
  answer = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hboxa), answer, TRUE, TRUE, 0);
  gtk_widget_set_size_request (answer, 50, -1);
  gtk_entry_set_alignment (GTK_ENTRY(answer), 1.0);
  gtk_widget_show(answer);
  dialog_ask = GTK_DIALOG (cask)->action_area;
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_ask), GTK_BUTTONBOX_END);

  ok = gtk_button_new_from_stock ("gtk-ok");
  gtk_dialog_add_action_widget (GTK_DIALOG (cask), ok, 1);
  gtk_widget_show(ok);

  if (gtk_dialog_run (GTK_DIALOG (cask)) == 1)
  {
    val = gtk_entry_get_text (GTK_ENTRY(answer));
    res = g_strdup_printf("%2s", (char *)val);
    gtk_widget_destroy (cask);
    return (res);
  }
  else
  {
    gtk_widget_destroy (cask);
    return ("  ");
  }
}
